import click
import helium
import dpath.util as dpath
import timeseries as ts


pass_service=click.make_pass_decorator(helium.Service)

@click.group()
def cli():
    """Operations on physical or virtual sensors.
    """
    pass

@cli.command()
@ts.options()
@pass_service
def timeseries(service, **kwargs):
    """List readings for the organization.

    Lists one page of readings for the organization.
    Readings can be filtered by PORT and by START and END date. Dates are given
    in ISO-8601 and may be one of the following forms:

    \b
    * YYYY-MM-DD - Example: 2016-05-05
    * YYYY-MM-DDTHH:MM:SSZ - Example: 2016-04-07T19:12:06Z
    """
    data = service.get_org_timeseries(**kwargs).get('data')
    ts.tabulate(data)


@cli.command()
@ts.format_option()
@ts.options(page_size=5000)
@pass_service
def dump(service, format, **kwargs):
    """Dumps timeseries data to files.

    Dumps the timeseries data for all sensors in the organization.

    One file is generated for each sensor with the sensor id as filename and the
    file extension based on the requested dump format

    This command takes the same arguments as the `timeseries` command, including
    the ability to filter by PORT, START and END date
    """
    sensors = dpath.values(service.get_sensors(), '/data/*/id')
    ts.dump(service, sensors, format, **kwargs)



import re
import ast
from setuptools import setup


setup(
    name='helium-commander',
    version='0.4.8',
    url='http://github.com/helium/helium-commander/',
    license='BSD',
    author='Marc Nijdam',
    author_email='marc@helium.com',
    description='A CLI and service wrapper for the Helium API',
    long_description=__doc__,
    packages=['helium', 'helium.commands'],
    include_package_data=True,
    platforms='all',
    install_requires=[
        'requests>=2.9',
        'dpath>=1.4',
        'futures>=3.0',
        'tablib>=0.11',
        'click>=6.6',
    ],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        "Topic :: Utilities",
        'Topic :: Software Development :: Libraries :: Python Modules'
    ],
    entry_points='''
        [console_scripts]
        helium=helium.cli:main
    '''
)



from concurrent import futures
import dpath.util as dpath
import helium
import click
import util
import csv
import json
import abc


def format_option():
    options = [
        click.option('--format', type=click.Choice(['csv', 'json']), default='csv',
                     help="the format of the readings")
    ]
    def wrapper(func):
        for option in options:
            func = option(func)
        return func
    return wrapper

def options(page_size=20):
    options = [
        click.option('--page-size', default=page_size,
                     help="the number of readings to get per request"),
        click.option('--port', multiple=True,
                     help="the port to filter readings on"),
        click.option('--start',
                     help="the start date to filter readings on"),
        click.option('--end',
                     help="the end date to filter readings on"),
    ]

    def wrapper(func):
        for option in options:
            func = option(func)
        return func
    return wrapper


def tabulate(result):
    if not result:
        click.echo('No data')
        return
    util.output(util.tabulate(result, [
        ('id', 'id'),
        ('timestamp', 'attributes/timestamp'),
        ('port', 'attributes/port'),
        ('value', 'attributes/value')
    ]))


def dump(service, sensors, format, **kwargs):
    with click.progressbar(length=len(sensors), label="Dumping", show_eta=False, width=50) as bar:
        with futures.ThreadPoolExecutor(max_workers=10) as executor:
            all_futures = []
            for sensor_id in sensors:
                future = executor.submit(_dump_one, service, sensor_id, format, **kwargs)
                future.add_done_callback(lambda f: bar.update(1))
                all_futures.append(future)
            result_futures = futures.wait(all_futures, return_when=futures.FIRST_EXCEPTION)
            for future in result_futures.done:
                future.result() # re-raises the exception

def _dump_one(service, sensor_id, format, **kwargs):
    filename = (sensor_id+'.'+format).encode('ascii', 'replace')
    with click.open_file(filename, "wb") as file:
        csv_mapping = {
            'id': 'id',
            'sensor': 'relationships/sensor/data/id',
            'timestamp': 'attributes/timestamp',
            'port': 'attributes/port',
            'value': 'attributes/value'
        }
        service = helium.Service(service.api_key, service.base_url)
        writer = _writer_for_format(format, file, mapping=csv_mapping)
        writer.process_timeseries(service, sensor_id, **kwargs)


#
# Writer classes
#

def _writer_for_format(format, file, **kwargs):
    if format == 'json':
        return _JSONWriter(file, **kwargs)
    elif format == 'csv':
        return _CSVWriter(file, **kwargs)

class _BaseWriter:
    __metaclass__ = abc.ABCMeta

    def __init__(self, file, **kwargs):
        self.file = file

    def process_timeseries(self, service, sensor_id, **kwargs):
        def json_data(json):
            return json['data'] if json else None
        # Get the first page
        res = service.get_sensor_timeseries(sensor_id, **kwargs)
        self.start()
        self.write_readings(json_data(res))
        while res != None:
            res = service.get_prev_page(res)
            self.write_readings(json_data(res))
        self.finish()

    def start(self):
        return

    @abc.abstractmethod
    def write_readings(self, readings):
        return

    def finish(self):
        return

class _CSVWriter(_BaseWriter):
    def __init__(self, file, **kwargs):
        super(_CSVWriter, self).__init__(file, **kwargs)
        self.mapping = kwargs['mapping']
        self.writer = csv.DictWriter(file, self.mapping.keys())

    def start(self):
        self.writer.writeheader()

    def write_readings(self, readings):
        if readings is None: return
        for reading in readings:
            row = { key: dpath.get(reading, path) for key, path in self.mapping.iteritems() }
            self.writer.writerow(row)


class _JSONWriter(_BaseWriter):
    def start(self):
        self.file.write('[')
        self.is_first_readings = True

    def write_readings(self, readings):
        if readings is None: return
        for reading in readings:
            if self.is_first_readings:
                self.is_first_readings = False
            else:
                self.file.write(',')
            self.file.write(json.dumps(reading))

    def finish(self):
        self.file.write(']')
